/*************** Practica 04. Implementacion clase Cola de clientes ***********
 *
 *  Nombre de Fichero: ColaClientes.cpp
 *
 *  Proposito: En este archivo se implementan los diferentes metodos de la
 *             clase cola de clientes
 *
 *****************************************************************************/
#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include <time.h>
#include "ColaClientes.h"

using namespace std;

/********************************************************/
/*Constructor de la la clase ColaClientes               */
/********************************************************/
ColaClientes::ColaClientes(void)
{
  srand(time(NULL));
  longitud = tam_max_cola=total_clientes = clientes_perdidos = 0;
  llega_en = tiempo_ult_suceso = tiempo_total_espera = tiempo_max_espera = 0;
  tam_total_cola = 0;
}
/*************** Rehusa **************************************************
 *
 *  Prototipo:    bool ColaClientes::ColaVacia(void)
 *
 *  Descripcion:  Comprueba si la cola esta vacia y devuelve true
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  void
 *
 *  Valor devuelto:
 *  bool    El cliente rehusa hacer cola
 *          true  -> La cola est vaca
 *          false -> La cola no est vacia
 *
 ************************************************************************/
bool ColaClientes::ColaVacia(void)
{
   bool vacia;
   
   if (cola.ColaVacia())
      vacia = true;
   else
      vacia = false;
      
   return vacia;
}
/*************** Rehusa **************************************************
 *
 *  Prototipo:    bool Cola::Rehusa()
 *
 *  Descripcion:  Si el nmero de clientes que hay en la cola es mayor de 4
 *                el cliente rehusa hacer cola en un 50% de los casos
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  void
 *
 *  Valor devuelto:
 *  bool    El cliente rehusa hacer cola
 *          true  -> El cliente rehusa hacer cola
 *          false -> El cliente no rehusa hacer cola
 *
 ************************************************************************/
bool ColaClientes::Rehusa()
{
  bool r = false;

  if (longitud > 5 && (rand() / Cola::Valor(RAND_MAX)) < 0.5)
  {
    cout << "Rehusa: " << longitud<<endl;
    r = true;
  }
  else cout << "No rehusa: " << longitud<<endl;
  return r;
}
/*************** Entrar **************************************************
 *
 *  Prototipo:    void Cola::Entrar(Cola::Valor tiempo_actual)
 *
 *  Descripcion:  Entra un nuevo cliente y se coloca en la cola
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  tiempo_actual   Cola::Valor  E   instante en el que entra el cliente
 *
 *  Valor devuelto:
 *  void
 *
 ************************************************************************/
void ColaClientes::Entrar(Cola::Valor tiempo_actual)
{
  total_clientes++;
  tam_total_cola += longitud * (tiempo_actual - tiempo_ult_suceso);
  tiempo_ult_suceso = tiempo_actual;
  if (Rehusa())
      clientes_perdidos++;
  else
   {
      cola.Encolar(tiempo_actual);
      longitud++;
      if (longitud > tam_max_cola) tam_max_cola = longitud;
   }
}
/*************** Salir ***************************************************
 *
 *  Prototipo:    void Cola::Salir(Cola::Valor tiempo_actual)
 *
 *  Descripcion:  Sale cliente de la cola
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  tiempo_actual   Cola::Valor  E   instante en el que sale el cliente
 *
 *  Valor devuelto:
 *  void
 *
 ************************************************************************/
void ColaClientes::Salir(Cola::Valor tiempo_actual)
{
  Cola::Valor tllegada, espera;

  tam_total_cola += longitud * (tiempo_actual - tiempo_ult_suceso);
  longitud --;
  tiempo_ult_suceso = tiempo_actual;
  cola.Desencolar(tllegada);
  espera = tiempo_actual - tllegada;
  if (espera > tiempo_max_espera)
      tiempo_max_espera = espera;
  tiempo_total_espera += espera;
}
/*************** LlegadaSiguiente ***************************************
 *
 *  Prototipo:    Cola::Valor Cola::LlegadaSiguiente(Cola::Valor tiempo_actual)
 *
 *  Descripcion:  Simula cuando llegar el siguiente cliente
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  tiempo_actual   Cola::Valor  E   hora actual
 *
 *  Valor devuelto:
 *  Cola::Valor  hora de llegada de un nuevo cliente
 *
 ************************************************************************/
Cola::Valor ColaClientes::LlegadaSiguiente(Cola::Valor tiempo_actual)
{
   Cola::Valor x, r;

   r = rand();
   x = -log(1 - (r/Cola::Valor(RAND_MAX))) / LLEGADA;
   x = tiempo_actual + (LLEGADA * exp(-LLEGADA * x));
   return x;
}
/********************** Siguiente ***************************************
 *
 *  Prototipo:    void Cola::Siguiente(Cola::Valor tiempo_actual)
 *
 *  Descripcion:  Guarda cuando llegar el siguiente cliente
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  tiempo_actual   Cola::Valor  E   hora actual
 *
 *  Valor devuelto:
 *  Cola::Valor  hora de llegada de un nuevo cliente
 *
 ************************************************************************/
void ColaClientes::Siguiente(Cola::Valor tiempo_actual)
{
 llega_en = LlegadaSiguiente(tiempo_actual);
}
/*************** LlegadaCliente *****************************************
 *
 *  Prototipo:    bool Cola::LlegaCliente(Cola::Valor tiempo_actual)
 *
 *  Descripcion:  Comprueba si ha llegado un cliente
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  tiempo_actual   Cola::Valor  E   hora actual
 *
 *  Valor devuelto:
 *  bool
 *          true  -> Ha llegado un cliente
 *          false -> No ha llegado un cliente
 *
 ************************************************************************/
bool ColaClientes::LlegaCliente(Cola::Valor tiempo_actual)
{
 bool res = false;

 if (tiempo_actual > llega_en)
     res = true;
 return res;
}
/*************** Estadsticas *******************************************
 *
 *  Prototipo:    void Cola::Estadisticas(Cola::Valor tiempo_actual)
 *
 *  Descripcion:  Muestra las estadsticas de la cola de clientes
 *
 *  Parametros:
 *  Nombre          Tipo        E/S  Descripcion
 *  ------          ----------  ---  ------------
 *  tiempo_actual   Cola::Valor  E   hora actual
 *
 *  Valor devuelto:
 *  void
 *
 ************************************************************************/
void ColaClientes::Estadisticas(Cola::Valor tiempo_actual)
{
  cout << endl << "ESTADSTICAS DE CLIENTES" << endl;
  cout << "\tClientes perdidos: " << clientes_perdidos << endl;
  cout << "\tTamao mximo de cola: " << tam_max_cola << endl;
  cout << "\tTamao medio de cola: " << tam_total_cola / tiempo_total_espera << endl;
  cout << "\tTiempo mximo de espera(seg): " << tiempo_max_espera << endl;
}

